/*
 * Developed by Hillstone Software. http://www.hillstone-software.com
 * HS POP3 lite Protocol Module public interface definitions
 */
#ifndef _HS_POP3_IF_H
#define _HS_POP3_IF_H

#ifndef WIN32
#define DWORD long long
#else
#define DWORD unsigned long
#endif

/* event callback function, called by POP3 module to report events to user code 
 * handle - user session handle 
 * ev - event id 
 * arg1, arg2 - depends on event, see HS POP3 manual */
typedef int event_callback_t(long handle, int ev, long arg1, long arg2);

/* timer callback function, handle - pop3 session handle */
typedef void timer_callback_t(void *arg);

/* start timer function, 
 * arg - pop3 session handle
 * timeout_ms - timeout value in milliseconds, 
 * callback -function (in pop3 module code) to call on timer expiry 
 * return - timer handle, to be used by pop3 to stop timer */
typedef long start_timer_t(unsigned long timeout_ms, void *arg, timer_callback_t *cb);

/* stop timer function, timer_handle - timer handle obtained from call to start_timer */
typedef void stop_timer_t(long timer_han);

/* api block */
typedef struct
{
    start_timer_t       *start_timer;   // start timer function
    stop_timer_t        *stop_timer;    // stop timer function
    event_callback_t    *event_cb;      // event callback function
} hs_pop3_api_t;

/* HS POP3 return codes */
typedef enum
{
    HS_POP3_RC_OK,                      // success
    HS_POP3_RC_NOTINIT,                 // HS POP3 module not initialised
    HS_POP3_RC_ALRINIT,                 // HS POP3 already initialised
    HS_POP3_RC_SOCKINIT_FAIL,           // socket layer initialisation failure
    HS_POP3_RC_INV_PAR,                 // invalid parameters
    HS_POP3_RC_NOUSER,                  // username missing
    HS_POP3_RC_NOPASS,                  // password missing
    HS_POP3_RC_NO_CTX,                  // exceeded maximum number of open sessions
    HS_POP3_RC_NONAME,                  // socket layer could not resolve server name
    HS_POP3_RC_TCPCONNFAIL,             // failed to connect to POP3 server
    HS_POP3_RC_TIMERFAIL,               // timer allocation failed

    HS_POP3_RC_NUM_RC,              	// internal significance to HS POP3 module only
} hs_pop3_rc_t;

#define HS_POP3_MAX_SRVNAME         80  // maximum size of server name string including null terminator
#define HS_POP3_MAX_USERNAME        80  // maximum size of user name string including null terminator
#define HS_POP3_MAX_PASSWORD        80  // maximum size of password string including null terminator

/* session parameters */
typedef struct
{
	/* either name or IP address may be specified, if not specified supply NULL pointer */
	/* if srv_name is NULL, try to connect using srv_ip - which is a string in the form x.x.x.x */
	/* if srv_name is not null, HS POP3 will try to resolve name to IP address */	
	unsigned char	srv_name[HS_POP3_MAX_SRVNAME];		// POP3 server name, null terminated string
	unsigned char   username[HS_POP3_MAX_USERNAME];     // POP3 user name, null terminated string
	unsigned char   password[HS_POP3_MAX_PASSWORD];     // POP3 password, null terminated string
	
	unsigned short	srv_port;		// POP3 server port

	int				delete_msgs;	// delete message from server after reception

    void			*user_data;		// user data association with this session (user session reference) not modified by POP3 module
} hs_pop3_session_t;

/* callback events */
typedef enum
{
	HS_POP3_USR_EV_DONE=1,			// Mail Session Complete, mail has been successfully received from POP3 server
	HS_POP3_USR_EV_DONE_MSG,        // Next message fully received from POP3 server
	HS_POP3_USR_EV_DONE_MSGBLK,	    // Next message block received of the same message
	HS_POP3_USR_EV_DONE_MSGHDR, // recdeived and parsed current message header
	HS_POP3_USR_EV_CLOSED,			// Socket layer closed TCP connection
	HS_POP3_USR_EV_CONNFAIL,		// TCP connection request failed
	HS_POP3_USR_EV_SRVERR,			// Session closed because of POP3 error response received from server
	HS_POP3_USR_EV_TIMEDOUT,		// Timed out waiting for server response
	HS_POP3_USR_EV_PROGRESS1,		// number of messages in mailbox and total size of mailbox
	HS_POP3_USR_EV_PROGRESS2,		// receiving current message number and number of bytes received
	HS_POP3_USR_EV_MSGID,			// message id indication, user code should return TRUE if it wants to receive this message
	HS_POP3_USR_EV_INTERR,			// internal error
} hs_pop3_usr_ev_t;

#define HS_POP3_MAX_PATH 257
#define HS_POP3_MAX_DATE 80
#define HS_POP3_MAX_SUBJ 257
#define HS_POP3_MAX_MSGID 71

typedef struct
{
	unsigned char	from[HS_POP3_MAX_PATH];		// FROM address
	unsigned char	date[HS_POP3_MAX_DATE];		// Date
	unsigned char	subj[HS_POP3_MAX_SUBJ];		// Subject
	unsigned char	msgid[HS_POP3_MAX_MSGID];	// message id
	int				hdrlen;						// length of message header
	DWORD			dwMsgLen;					// full message length including headers
} hs_pop3_msg_t;

/*
 * Initialise HS POP3 Library
 */
extern int	HsPop3Init(hs_pop3_api_t *api);

/*
 * Release all HS POP3 resources and clean up
 */
extern int	HsPop3Destroy(void);

/*
 * Open Pop3 Session and receive mail if any
 */
extern int HsPop3GetMail(hs_pop3_session_t *s, long *session_handle);

/*
 * Translate integer HS POP3 return code into string
 */
extern unsigned char *HsPop3GetErrStr(int rc);

/* Abort mail session */
extern int HsPop3Abort(long session_handle);

#endif /* _HS_POP3_IF_H */
